/*  Program modified for display temperature on mini web server
Accesories used are: MCU PIC18F2620;
                     Temperature sensor DS18s20;
                     ENC28j60;
                     
 * Test configuration:
     MCU:             PIC18F4620
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39631E.pdf
     Dev.Board:           PIC-Ready1
                          http://www.mikroe.com/products/view/177/pic-ready-prototype-board/
                          http://www.mikroe.com/products/view/305/pic-ready1-board/

     Oscillator:      HS, 8.000000 MHz
     ext. modules:     ard
                          http://www.mikroe.com/eng/products/view/14/serial-ethernet-board/

     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/en/compilers/mikroc/pro/pic/

 /INT : RB1 (marron)
 /RST : RC0 (vert)
 /CS  : RC1 (blanc)
 MISO : RC4 (rose)
 MOSI : RC5 (gris)
 SCK  : RC3 (jaune)
 
 * NOTES:
     - Since the ENC28J60 doesn't support auto-negotiation, full-duplex mode is
       not compatible with most switches/routers.  If a dedicated network is used
       where the duplex of the remote node can be manually configured, you may
       change this configuration.  Otherwise, half duplex should always be used.
       
Temperature sensor DS18s20
http://www.mikroe.com/products/view/280/ds1820-temperature-sensor/
Branched on port E pin 0
*/
// Configuration LCD
sbit LCD_RS at LATD2_bit;
sbit LCD_EN at LATD0_bit;
sbit LCD_D4 at LATD1_bit;
sbit LCD_D5 at LATD3_bit;
sbit LCD_D6 at LATD5_bit;
sbit LCD_D7 at LATD7_bit;

sbit LCD_RS_Direction at TRISD2_bit;
sbit LCD_EN_Direction at TRISD0_bit;
sbit LCD_D4_Direction at TRISD1_bit;
sbit LCD_D5_Direction at TRISD3_bit;
sbit LCD_D6_Direction at TRISD5_bit;
sbit LCD_D7_Direction at TRISD7_bit;
#include "MikroC.h"

#include "Lib_1Wire_DS18s20.h"

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------

void Mesure()
{
 Reading_Temperature_DS18s20();
   //Delay_ms(250);
}

char uart_rd;
char  buffer[32];

void Display_UART()
{
 char  T_buffer[15];

 FloatToStr(temperature_DS18s20, T_buffer);

 UART1_Write_Text(T_buffer);
 UART1_Write_Text("'C\r");
}

sbit SPI_Ethernet_Rst at RC0_bit;
sbit SPI_Ethernet_CS at RC1_bit;

sbit SPI_Ethernet_Rst_Direction at TRISC0_bit;
sbit SPI_Ethernet_CS_Direction at TRISC1_bit;

#define SPI_Ethernet_HALFDUPLEX     0
#define SPI_Ethernet_FULLDUPLEX     1

/*
 * definition of user functions called by the ehternet library.
 * these functions are used for tcp and udp handling.
 * we do not need them in this example, but, nevertheless they must be defined,
 * as described in mikroC help.
 *
 */

const unsigned short TEMP_RESOLUTION = 12;
char *text = "000,0";
//unsigned char ch,ch1,ch2,ch3;

const unsigned char httpHeader[] = "HTTP/1.1 200 OK\nContent-type: " ;  // HTTP header
const unsigned char httpMimeTypeHTML[] = "text/html\n\n" ;              // HTML MIME type
const unsigned char httpMimeTypeScript[] = "text/plain\n\n" ;           // TEXT MIME type
unsigned char httpMethod[] = "GET /";                                   // supported http method

#define __IP_LOCAL__ "192.168.0.170"
#define __hebergement__   "http://olivier.fournet.free.fr/electronique/Display_temperature_on_mini_web_server_v2/js/"
#define __Temp_between_measurements__     "1"
#define __Temp_between_measurements_in_Second__  1
/*
 * web page, splited into 2 parts :
 * when coming short of ROM, fragmented data is handled more efficiently by linker
 *
 * this HTML page calls the boards to get its status, and builds itself with javascript
 */
const   char    *indexPage = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\
<meta http-equiv=\"refresh\" content=\"" __Temp_between_measurements__ ";url=http://" __IP_LOCAL__ "/\">\
<HTML><HEAD>\
<script language=\"javascript\" type=\"text/javascript\" src=\"" __hebergement__ "Interactivity_init.js\"></script>\
<script language=\"javascript\" type=\"text/javascript\">init_Interactivity();</script>\
<title>ENC28J60 & PIC18F4620 & DS18s20 - Flot Examples: Interactivity</title>\
</HEAD><BODY>\
<center>WEB Server designed for PIC18F4620, ENC28J60 and DS18s20 (Flot Examples: Interactivity),<p>\
<p>avec " __COMPILER__ "<p>Compilation  " __TIME__ " le " __DATE__"<p></center>\
<hr size=2 align=center width=60%>\
<center><a href=/>REFRESH</a></center>\
<script src=/s></script>\
<p><hr size=2>\
<center>TEMPERATURE: <h1><script>document.write(T_)</script>C</h1></center>\
<p>\
This is HTTP request=<script>document.write(REQ)</script> , Seconds=<script>document.write(Seconds)</script>\
<p><hr size=2><br>\
<script language=\"javascript\" type=\"text/javascript\">Interactivity();</script>\
</BODY></HTML>";
/***********************************
 * RAM variables
 */
unsigned char   myMacAddr[6] = {0x00, 0x14, 0xA5, 0x76, 0x19, 0x3f};   // my MAC address
//unsigned char   myIpAddr[4] = {0, 0, 0, 0};        // my IP address
unsigned char   myIpAddr[4] = {192, 168, 0, 170};       // my IP address
unsigned char   getRequest[15];                                        // HTTP request buffer
unsigned char   dyna[31];                                              // buffer for dynamic response
unsigned long   httpCounter = 0; // counter of HTTP requests
unsigned long   Sample = 0;
char txt_IP[4];

typedef struct
{
  unsigned canCloseTCP: 1;
  unsigned isBroadcast: 1;
} TEthPktFlags;

#define NB_SAMPLES    10
float temperature[NB_SAMPLES];// = { 0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.01,1.02,1.03,1.04 };
float time[NB_SAMPLES];// = { -0.1,2,3,4,5,6,7,8,9,10,11,12,13,14 };
float x_min = 0.0, x_max = 0.0;
float y_min = 0.0, y_max = 0.0;
float Seconds;
	        
/*******************************************
 * functions
 */

/*
 * put the constant string pointed to by s to the ENC transmit buffer
 */
#define putConstString  SPI_Ethernet_putConstString
/*
unsigned int    putConstString(const char *s)
        {
        unsigned int ctr = 0 ;

        while(*s)
                {
                SPI_Ethernet_putByte(*s++) ;
                ctr++ ;
                }
        return(ctr) ;
        }
*/
/*
 * put the string pointed to by s to the ENC transmit buffer
 */
#define putString  SPI_Ethernet_putString
/*
unsigned int    putString(char *s)
        {
        unsigned int ctr = 0 ;

        while(*s)
                {
                SPI_Ethernet_putByte(*s++) ;
                ctr++ ;
                }
        return(ctr) ;
        }
*/
/*
 * this function is called by the library
 * the user accesses to the HTTP request by successive calls to SPI_Ethernet_getByte()
 * the user puts data in the transmit buffer by successive calls to SPI_Ethernet_putByte()
 * the function must return the length in bytes of the HTTP reply, or 0 if nothing to transmit
 *
 * if you don't need to reply to HTTP requests,
 * just define this function with a return(0) as single statement
 *
 */
unsigned int    SPI_Ethernet_UserTCP(unsigned char *remoteHost, unsigned int remotePort,
                         unsigned int localPort, unsigned int reqLength, char *canClose)
        {
        unsigned int    len = 0 ;                               // my reply length
        unsigned int    i ;                                     // general purpose integer

        if(localPort != 80)                                     // I listen only to web request on port 80
                {
                return(0) ;
                }

        // get 10 first bytes only of the request, the rest does not matter here
        for(i = 0 ; i < 10 ; i++)
                {
                getRequest[i] = SPI_Ethernet_getByte() ;
                }
        getRequest[i] = 0 ;

        if(memcmp(getRequest, httpMethod, 5))                   // only GET method is supported here
                {
                return(0) ;
                }

        httpCounter++ ;                                         // one more request done

        if(getRequest[5] == 's')                                // if request path name starts with s, store dynamic data in transmit buffer
        {
                 Display_UART();
                // the text string replied by this request can be interpreted as javascript statements
                // by browsers

                len = putConstString(httpHeader) ;              // HTTP header
                len += putConstString(httpMimeTypeScript) ;     // with text MIME type

                // add T value to reply
                len += putConstString("T_=");
                FloatToStr(temperature_DS18s20, dyna);
                //IntToStr(i_100x_temperature_DS18s20, dyna);
                len += putString(dyna);
                len += putConstString("; ");///100.0;");
         
                // add HTTP requests counter to reply
                IntToStr(httpCounter, dyna);                   // convert httpCounter value into string
                len += putConstString("var REQ=");
                len += putString(dyna);
                len += putConstString("; ");
                
                // add Sec counter to reply
                Seconds = __Temp_between_measurements_in_Second__ * (float)httpCounter;
                FloatToStr(Seconds, dyna);                   // convert Sec value into string
                len += putConstString("var Seconds=");
                len += putString(dyna);
                len += putConstString("; ");
                /*
                var color_T = "#FF0000";
	        var label_t = "T(x)";
	        var x_min = -0.5, x_max =  14.5, y_min = -0.5, y_max =  1.5;
                */
                x_min = x_max = Seconds;
                y_min = y_max = temperature_DS18s20;
                
                for(i = 1 ; i < NB_SAMPLES ; i++)
                {
                 time[i-1] = time[i];
                 temperature[i-1] = temperature[i];
                 if( time[i] < x_min ) x_min = time[i];
                 if( time[i] > x_max ) x_max = time[i];
	         if( temperature[i] < y_min ) y_min = temperature[i];
	         if( temperature[i] > y_max ) y_max = temperature[i];
                }

                temperature[NB_SAMPLES-1] = temperature_DS18s20;
                time[NB_SAMPLES-1] = Seconds;
         
                len += putConstString("var color_T = \"#FF0000\"; ");
	        len += putConstString("var label_t = \"T(x)\"; ");

	        len += putConstString("var x_min = ");
	        FloatToStr(x_min, dyna);
                len += putString(dyna);
	        len += putConstString(", x_max = ");
	        FloatToStr(x_max, dyna);
                len += putString(dyna);
                len += putConstString(", y_min = ");
                FloatToStr(y_min, dyna);
                len += putString(dyna);
                len += putConstString(", y_max = ");
                FloatToStr(y_max, dyna);
                len += putString(dyna);
	        len += putConstString("; ");
	        
                if(Sample > NB_SAMPLES)
                {
                  //len += putConstString("var temperature = [[-0.1,0.1],[2,0.2],[3,0.3],[4,0.4],[5,0.5],[6,0.6],[7,0.7],[8,0.8],[9,0.9],[10,1],[11,1.01],[12,1.02],[13,1.03],[14,1.04]];");
                  len += putConstString("var temperature = [");
                  for(i = 0 ; i < NB_SAMPLES ; i++)
                  {
                   len += putConstString("[");
                   FloatToStr(time[i], dyna);
                   len += putString(dyna);
                   len += putConstString(",");
                   FloatToStr(temperature[i], dyna);
                   len += putString(dyna);
                   len += putConstString("],");
                  }
                  len += putConstString("];");
                 }
         }

        if(len == 0)                                            // what do to by default
                {
                len =  putConstString(httpHeader) ;             // HTTP header
                len += putConstString(httpMimeTypeHTML) ;       // with HTML MIME type
                len += putConstString(indexPage) ;              // HTML page first part
                }

        return(len) ;                                           // return to the library with the number of bytes to transmit
        }

/*
 * this function is called by the library
 * the user accesses to the UDP request by successive calls to SPI_Ethernet_getByte()
 * the user puts data in the transmit buffer by successive calls to SPI_Ethernet_putByte()
 * the function must return the length in bytes of the UDP reply, or 0 if nothing to transmit
 *
 * if you don't need to reply to UDP requests,
 * just define this function with a return(0) as single statement
 *
 */

unsigned int    SPI_Ethernet_UserUDP(unsigned char *remoteHost, unsigned int remotePort,
                         unsigned int localPort, unsigned int reqLength, char *canClose)
{
 return 0 ;                      // back to the library with the length of the UDP reply
}

/* ---- IP Auto ----- */
/*
unsigned int SPI_Ethernet_UserTCP(unsigned char *remoteHost, unsigned int remotePort, unsigned int localPort, unsigned int reqLength, TEthPktFlags *flags)
{
 unsigned int    len;

 if(localPort != 80)
 {
  return(0) ;
 }
}

unsigned int SPI_Ethernet_UserUDP(unsigned char *remoteHost, unsigned int remotePort, unsigned int destPort, unsigned int reqLength, TEthPktFlags *flags)
{
        unsigned int    len ;

        ByteToStr(remoteHost[0], dyna) ;                // prvi bit IP adrese
        dyna[3] = '.' ;
        ByteToStr(remoteHost[1], dyna + 4) ;            // drugi
        dyna[7] = '.' ;
        ByteToStr(remoteHost[2], dyna + 8) ;            // treci
        dyna[11] = '.' ;
        ByteToStr(remoteHost[3], dyna + 12) ;           // cetvrti

        dyna[15] = ':' ;

        WordToStr(remotePort, dyna + 16) ;
        dyna[21] = '[' ;
        WordToStr(destPort, dyna + 22) ;
        dyna[27] = ']' ;
        dyna[28] = 0 ;

        len = 28 + reqLength;

        SPI_Ethernet_putBytes(dyna, 28) ;

        while(reqLength--)
        {
         SPI_Ethernet_putByte(toupper(SPI_Ethernet_getByte())) ;
        }

 return(len);
}
*/
/*
 * main entry
 */

void main()
{
 unsigned int i;
 CMCON |=7;
 ADCON0= 0;
 ADCON1 = 0x0F ;                 // ADC convertors will be used
 ADCON2 = 0x40;
 /*
 PORTA = 0 ;
 TRISA0_bit = 1;                  // set PORTA as input for ADC
  */
 PORTB = 0 ;
 TRISB = 0xff ;                  // set PORTB as input for buttons

 PORTA = 0;
 TRISA = 0x01;
 
 Sample = 0;
 
 for(i = 0 ; i < NB_SAMPLES ; i++)
 {
  time[i] = 0.0;
  temperature[i] = 0.0;
 }
 
 Lcd_Init();
Lcd_Cmd(_LCD_CLEAR);
Lcd_Cmd(_LCD_CURSOR_OFF );
Lcd_Out(1, 1, "Status:");

 Reading_Temperature_DS18s20();
  Delay_ms(1000);                 // Wait Reading temprature
  Reading_Temperature_DS18s20();

  UART1_Init(57600);          // Initialize UART module at 9600 bps
  Delay_ms(100);             // Wait for UART module to stabilize

  Display_Version_UART();
        /*
         * initialize ethernet board
         * start ENC28J60 with :
         * reset bit on RC0
         * CS bit on RC1
         * my MAC & IP address
         * full duplex
         */
         /*
        SPI1_Init();
 SPI_Ethernet_Init(myMacAddr, myIpAddr,0);

 SPI_Ethernet_initDHCP(5);
 memcpy(myIpAddr, SPI_Ethernet_getIpAddress(),4);
      */

        SPI1_Init();
        SPI_Ethernet_Init(myMacAddr, myIpAddr, SPI_Ethernet_FULLDUPLEX);

 Lcd_Out(1, 1, "IP :    ");
 ByteToStr(myIpAddr[0], txt_IP);
 Lcd_Out(2, 1, txt_IP);
 Lcd_Out_CP(".");
 ByteToStr(myIpAddr[1], txt_IP);
 Lcd_Out_CP(txt_IP);
 Lcd_Out_CP(".");
 ByteToStr(myIpAddr[2], txt_IP);
 Lcd_Out_CP(txt_IP);
 Lcd_Out_CP(".");
 ByteToStr(myIpAddr[3], txt_IP);
 Lcd_Out_CP(txt_IP);
 
        while(1)                            // endless loop
        {
         //--- Perform temperature reading
               // Reading_Temperature_DS18s20();
         Mesure();
         Sample++;

         //--- Format and display result on UART
         Display_UART();

                /*
                 * add your stuff here if needed
                 * SPI_Ethernet_doPacket() must be called as often as possible
                 * otherwise packets could be lost
                 */
                 SPI_Ethernet_doPacket() ;   // process incoming Ethernet packets
         }
        }